class Error(Exception):
	pass

class NumTooLowError(Error):
	def __init__(self, value, message="Value was too low."):
			self.value = value
			self.message = message
			super().__init__(self.message)
	
	def __str__(self):
			return f'{self.value}: {self.message}'

class NumTooHighError(Error):
	def __init__(self, value, message="Value was too high."):
		self.value = value
		self.message = message
		super().__init__(self.message)
	
	def __str__(self):
		return f'{self.value}: {self.message}'

class NumNotInRangeError(Error):
	def __init__(self, nrange, value, message="Value not in range "):
		if not type(nrange) == list:
			raise TypeError("nrange must be a list.")
	
		elif nrange[0] < value < nrange[1]: 
			raise Exception("Hey, don't misuse exceptions!")

		else: self.nrange = nrange
		self.value = value
		self.message = message
		super().__init__(self.message)
	
	def __str__(self):
		return f'{self.value}: {self.message} ({self.nrange[0]}, {self.nrange[1]})'



def checkstr(ost):
		if type(ost) == str:
			return True
		else: return False

def rev(ost):
	if checkstr(ost):
		revved = ost[::-1]
		return revved
	else: raise SyntaxError


def ascii_convert(ost):
	if checkstr(ost):
		rst = ""
		for char in ost:
			rst += ("-"+str(ord(char)))
		return rst.lstrip("-")
	else: raise SyntaxError

def caesarcipher(ost, key=1):
	if checkstr(ost):
		if type(key) == int:
			rst = ""
			for char in ost:
				rst += chr(ord(char) + key)
			return rst
		else: raise TypeError
	else: raise SyntaxError

def weirdcase(ost):
    if checkstr(ost):
        rst = ""
        l = list(ost.casefold())
        for i in range(len(l)):
            if i % 2 == 0:
                 rst += l[i].upper()
            else:
                rst += l[i].lower()
        return rst

def encode(ost, key=1):
	if checkstr(ost):
		if type(key) == int:
			rst = rev(ascii_convert(weirdcase(caesarcipher(ost, key))))
			return rst
		else: raise TypeError
	else: raise SyntaxError

def checkpalindrome(ost):
	if checkstr(ost):
		if list(ost) == list(rev(ost)):
			return True
		else: return False
